// Servant which implements the RecycleBroker::CustomerAdmin interface.

#include <stdlib.h>
#include <iostream.h>
#include "RecycleBroker_CustomerAdmin_i.h"
#include "RecycleBroker_Customer_i.h"
#include "ObjectIdMapper.h"
#include "POAMapper.h"
#include "DBBase.h"


// _create() -- create a new servant.
// Hides the difference between direct inheritance and tie servants
// For direct inheritance, simple create and return an instance of the servant.
// For tie, creates an instance of the tied class and the tie, return the tie.
//
POA_RecycleBroker::CustomerAdmin*
RecycleBroker_CustomerAdmin_i::_create(
    CORBA::Boolean          hasAdminPrivileges,
    RecycleBroker::KeyType  branch_id
)
{
    return new RecycleBroker_CustomerAdmin_i(hasAdminPrivileges, branch_id);
}


// RecycleBroker_CustomerAdmin_i constructor
RecycleBroker_CustomerAdmin_i::RecycleBroker_CustomerAdmin_i(
    CORBA::Boolean          hasAdminPrivileges,
    RecycleBroker::KeyType  branch_id
) :
    m_hasAdminPrivileges(hasAdminPrivileges),
    m_branch_id(branch_id)
{
}


// ~RecycleBroker_CustomerAdmin_i destructor.
RecycleBroker_CustomerAdmin_i::~RecycleBroker_CustomerAdmin_i()
{
}


// create() -- Implements IDL operation "RecycleBroker::CustomerAdmin::create".
RecycleBroker::Customer_ptr
RecycleBroker_CustomerAdmin_i::create(
    const RecycleBroker::CustomerDetails& initialData,
    RecycleBroker::KeyType_out      customer_id
) IT_THROW_DECL((
    CORBA::SystemException,
    RecycleBroker::NoPermission
))
{
    cout << "RecycleBroker_CustomerAdmin_i::create(): called." << endl;

    if (!m_hasAdminPrivileges) { throw RecycleBroker::NoPermission(); }

    CORBA::String_var               rep_idV;
    PortableServer::ObjectId_var    oidV;
    PortableServer::ServantBase_var servantV;
    PortableServer::POA_var         customer_poaV;
    CORBA::Object_var               objV;
    RecycleBroker::Customer_var     refV;

    // Lifecycle: Basic POA 

    // Create a DB record for the new CORBA object
    customer_id = global_db->create_customer_details(
                                 initialData,
                                 m_branch_id
                             );

    // Construct an 'ObjectId' from the database key, 'customer_id'
    rep_idV = RecycleBroker::_tc_Customer->id();
    oidV = ObjectIdMapper::make_ObjectId(rep_idV.in(), customer_id);

    cout << "\tkey = " << customer_id << endl;

    // Create a 'Customer_i' servant
    servantV = new RecycleBroker_Customer_i(customer_id);

    // Get the POA for 'Customer' CORBA objects
    customer_poaV = servantV->_default_POA();

    // Create a 'Customer' object reference
    customer_poaV->activate_object_with_id(oidV, servantV.in());
    objV = customer_poaV->id_to_reference(oidV.in() );
    refV = RecycleBroker::Customer::_narrow(objV.in() );

    return refV._retn();
}


// find() -- Implements IDL operation "RecycleBroker::CustomerAdmin::find".
//
RecycleBroker::Customer_ptr
RecycleBroker_CustomerAdmin_i::find(
    RecycleBroker::KeyType          customer_id
) IT_THROW_DECL((
    CORBA::SystemException,
    RecycleBroker::NoPermission,
    RecycleBroker::NotFound
))
{
    CORBA::String_var               rep_idV;
    PortableServer::ObjectId_var    oidV;
    PortableServer::ServantBase_var servantV;
    PortableServer::POA_var         customer_poaV;
    CORBA::Object_var               objV;
    RecycleBroker::Customer_var     refV;

    cout << "RecycleBroker_CustomerAdmin_i::find("
         << customer_id << ") called" << endl;

    if (!m_hasAdminPrivileges) { throw RecycleBroker::NoPermission(); }

    // Case: Basic POA

    // Construct an 'ObjectId' from the database key, 'customer_id'
    rep_idV = RecycleBroker::_tc_Customer->id();
    oidV = ObjectIdMapper::make_ObjectId(rep_idV.in(), customer_id);

    // Get the POA for 'Customer' CORBA objects
    customer_poaV = POAMapper::getPOA(rep_idV.in() );

    try {
        // Look up the 'oid' in the 'Active Object Map'
        objV = customer_poaV->id_to_reference(oidV.in() );
    }
    catch (PortableServer::POA::ObjectNotActive& ) {
        objV = CORBA::Object::_nil();
    }

    if (CORBA::is_nil(objV.in()) ) {
        // Object not active --> check validity of 'customer_id'
        RecycleBroker::CustomerDetailsFull_var  dataV;
        if (global_db->find_customer_details(customer_id, dataV.out()) )
        {
            // Create a 'Customer_i' servant
            servantV = new RecycleBroker_Customer_i(dataV.in() );

            // Create a 'Customer' object reference
            customer_poaV->activate_object_with_id(oidV, servantV.in() );
            objV = customer_poaV->id_to_reference(oidV.in() );
        }
        else {
            throw RecycleBroker::NotFound();
        }
    }

    refV = RecycleBroker::Customer::_narrow(objV.in() );
    return refV._retn();
}


// find_by_name() -- Implements IDL operation "RecycleBroker::CustomerAdmin::find_by_name".
//
RecycleBroker::CustomerIdSeq*
RecycleBroker_CustomerAdmin_i::find_by_name(
    const char*                     name
) IT_THROW_DECL((
    CORBA::SystemException,
    RecycleBroker::NoPermission,
    RecycleBroker::NotFound
))
{
    if (!m_hasAdminPrivileges) { throw RecycleBroker::NoPermission(); }

    RecycleBroker::CustomerIdSeq* _resultP
        = global_db->find_customer_details_by_name(name);

    return _resultP;
}

PortableServer::POA_ptr
RecycleBroker_CustomerAdmin_i::_default_POA()
{
    CORBA::String_var rep_idV = RecycleBroker::_tc_CustomerAdmin->id();
    return POAMapper::getPOA(rep_idV.in() );
}

