// Servant which implements the RecycleBroker::WasteItemAdmin interface.

#include <stdlib.h>
#include <iostream.h>
#include "RecycleBroker_WasteItemAdmin_i.h"
#include "RecycleBroker_WasteItem_i.h"
#include "ObjectIdMapper.h"
#include "POAMapper.h"
#include "DBBase.h"


// _create() -- create a new servant.
POA_RecycleBroker::WasteItemAdmin*
RecycleBroker_WasteItemAdmin_i::_create(
    CORBA::Boolean          hasAdminPrivileges,
    RecycleBroker::KeyType  branch_id,
    RecycleBroker::KeyType  customer_id
)
{
    return new RecycleBroker_WasteItemAdmin_i(
                   hasAdminPrivileges,
                   branch_id,
                   customer_id
               );
}


// RecycleBroker_WasteItemAdmin_i constructor
RecycleBroker_WasteItemAdmin_i::RecycleBroker_WasteItemAdmin_i(
    CORBA::Boolean          hasAdminPrivileges,
    RecycleBroker::KeyType  branch_id,
    RecycleBroker::KeyType  customer_id
) :
    m_hasAdminPrivileges(hasAdminPrivileges),
    m_branch_id(branch_id),
    m_customer_id(customer_id)
{
}

// Create a 'WasteItemAdmin' object with no privileges.
RecycleBroker_WasteItemAdmin_i::RecycleBroker_WasteItemAdmin_i(
    RecycleBroker::KeyType  branch_id
) :
    m_hasAdminPrivileges(0),
    m_branch_id(branch_id),
    m_customer_id(-1)
{
}


// ~RecycleBroker_WasteItemAdmin_i destructor.
//
RecycleBroker_WasteItemAdmin_i::~RecycleBroker_WasteItemAdmin_i()
{
}


// create() -- Implements IDL operation "RecycleBroker::WasteItemAdmin::create".
RecycleBroker::WasteItem_ptr
RecycleBroker_WasteItemAdmin_i::create(
    const RecycleBroker::WasteItemDetails& initialData,
    RecycleBroker::KeyType_out      wasteitem_id
) IT_THROW_DECL((
    CORBA::SystemException,
    RecycleBroker::NoPermission
))
{
    cout << "RecycleBroker_WasteItemAdmin_i::create(): called." << endl;

    if (!m_hasAdminPrivileges) { throw RecycleBroker::NoPermission(); }

    CORBA::String_var               rep_idV;
    PortableServer::ObjectId_var    oidV;
    PortableServer::ServantBase_var servantV;
    PortableServer::POA_var         wasteitem_poaV;
    CORBA::Object_var               objV;
    RecycleBroker::WasteItem_var     refV;

    // Lifecycle: Basic POA 

    // Create a DB record for the new CORBA object
    wasteitem_id = global_db->create_wasteitem_details(
                                 initialData,
                                 m_branch_id,
                                 m_customer_id
                             );

    // Construct an 'ObjectId' from the database key, 'wasteitem_id'
    rep_idV = RecycleBroker::_tc_WasteItem->id();
    oidV = ObjectIdMapper::make_ObjectId(rep_idV.in(), wasteitem_id);

    cout << "\tkey = " << wasteitem_id << endl;

    // Create a 'WasteItem_i' servant
    servantV = new RecycleBroker_WasteItem_i(wasteitem_id);

    // Get the POA for 'WasteItem' CORBA objects
    wasteitem_poaV = servantV->_default_POA();

    // Create a 'WasteItem' object reference
    wasteitem_poaV->activate_object_with_id(oidV, servantV.in());
    objV = wasteitem_poaV->id_to_reference(oidV.in() );
    refV = RecycleBroker::WasteItem::_narrow(objV.in() );

    return refV._retn();
}


// find() -- Implements IDL operation "RecycleBroker::WasteItemAdmin::find".
//
RecycleBroker::WasteItem_ptr
RecycleBroker_WasteItemAdmin_i::find(
    RecycleBroker::KeyType          wasteitem_id
) IT_THROW_DECL((
    CORBA::SystemException,
    RecycleBroker::NotFound
))
{
    CORBA::String_var               rep_idV;
    PortableServer::ObjectId_var    oidV;
    PortableServer::ServantBase_var servantV;
    PortableServer::POA_var         wasteitem_poaV;
    CORBA::Object_var               objV;
    RecycleBroker::WasteItem_var     refV;

    cout << "RecycleBroker_WasteItemAdmin_i::find("
         << wasteitem_id << ") called" << endl;

    // Case: Basic POA

    // Construct an 'ObjectId' from the database key, 'wasteitem_id'
    rep_idV = RecycleBroker::_tc_WasteItem->id();
    oidV = ObjectIdMapper::make_ObjectId(rep_idV.in(), wasteitem_id);

    // Get the POA for 'WasteItem' CORBA objects
    wasteitem_poaV = POAMapper::getPOA(rep_idV.in() );

    try {
        // Look up the 'oid' in the 'Active Object Map'
        objV = wasteitem_poaV->id_to_reference(oidV.in() );
    }
    catch (PortableServer::POA::ObjectNotActive& ) {
        objV = CORBA::Object::_nil();
    }

    if (CORBA::is_nil(objV.in()) ) {
        // Object not active --> check validity of 'wasteitem_id'
        RecycleBroker::WasteItemDetailsFull*  dataP;
        if (global_db->find_wasteitem_details(wasteitem_id, dataP) )
        {
            // Create a 'WasteItem_i' servant
            servantV = new RecycleBroker_WasteItem_i(*dataP);
            delete dataP;

            // Create a 'WasteItem' object reference
            wasteitem_poaV->activate_object_with_id(oidV, servantV.in() );
            objV = wasteitem_poaV->id_to_reference(oidV.in() );
        }
        else {
            delete dataP;
            throw RecycleBroker::NotFound();
        }
    }

    refV = RecycleBroker::WasteItem::_narrow(objV.in() );
    return refV._retn();
}


// find_by_waste() -- Implements IDL operation "RecycleBroker::WasteItemAdmin::find_by_waste".
//
RecycleBroker::WasteItemIdSeq*
RecycleBroker_WasteItemAdmin_i::find_by_waste(
    RecycleBroker::WasteType        waste
) IT_THROW_DECL((
    CORBA::SystemException,
    RecycleBroker::NotFound
))
{
    if (!global_db) { throw CORBA::PERSIST_STORE(); }

    RecycleBroker::WasteItemIdSeq*  _result = 0;

    if (!global_db->find_wasteitem_by_waste(waste, _result))
    {
        delete _result;
        throw RecycleBroker::NotFound();
    }

    return _result;
}


// find_by_branch() -- Implements IDL operation "RecycleBroker::WasteItemAdmin::find_by_branch".
//
RecycleBroker::WasteItemIdSeq*
RecycleBroker_WasteItemAdmin_i::find_by_branch(
    RecycleBroker::KeyType          branch_id
) IT_THROW_DECL((
    CORBA::SystemException,
    RecycleBroker::NotFound
))
{
    if (!global_db) { throw CORBA::PERSIST_STORE(); }

    RecycleBroker::WasteItemIdSeq*  _result = 0;

    if (!global_db->find_wasteitem_by_branch(branch_id, _result))
    {
        delete _result;
        throw RecycleBroker::NotFound();
    }

    return _result;
}


// find_all() -- Implements IDL operation "RecycleBroker::WasteItemAdmin::find_all".
//
RecycleBroker::WasteItemIdSeq*
RecycleBroker_WasteItemAdmin_i::find_all() IT_THROW_DECL((
    CORBA::SystemException
))
{
    if (!global_db) { throw CORBA::PERSIST_STORE(); }

    RecycleBroker::WasteItemIdSeq*  _result = 0;

    if (!global_db->find_all_wasteitem(_result))
    {
        delete _result;
        throw RecycleBroker::NotFound();
    }

    return _result;
}


// get_details() -- Implements IDL operation "RecycleBroker::WasteItemAdmin::get_details".
//
RecycleBroker::WasteItemDetailsSeq*
RecycleBroker_WasteItemAdmin_i::get_details(
    const RecycleBroker::WasteItemIdSeq& id_seq
) IT_THROW_DECL((
    CORBA::SystemException,
    RecycleBroker::NotFound
))
{
    if (!global_db) { throw CORBA::PERSIST_STORE(); }

    RecycleBroker::WasteItemDetailsSeq*  _result = 0;

    if (!global_db->find_wasteitem_details(id_seq, _result))
    {
        delete _result;
        throw RecycleBroker::NotFound();
    }

    return _result;
}

PortableServer::POA_ptr
RecycleBroker_WasteItemAdmin_i::_default_POA()
{
    CORBA::String_var rep_idV = RecycleBroker::_tc_WasteItemAdmin->id();
    return POAMapper::getPOA(rep_idV.in() );
}

