// C++

#include <unistd.h>
#include <iostream.h>
#include <iomanip.h>
#include <corba.h>
#include <CosEventComm_s.hh>
#include <CosEventChannelAdmin_c.hh>

class PurePullConsumer : public POA_CosEventComm::PullConsumer {

public:
    PurePullConsumer(CosEventComm::PullSupplier_ptr pullSupplier,
            PortableServer::POA_ptr poa) : _pullSupplier(pullSupplier),
            _poa(poa) { }

    void start() {

        CORBA::Any* anyVal;
        CORBA::ULong longVal;
        CORBA::Short shortVal;
        CORBA::Double doubleVal;
        char* stringVal;
        CORBA::Boolean hasEvent = 0;

        cout << "Consumer in start of thread, sleeping..." << endl;
        sleep(2);
        while (1) {
            try {
                while (!hasEvent) {
                    sleep(1);
                    cout << "Consumer calling try_pull" << endl;
                    try {
                        anyVal = _pullSupplier->try_pull(hasEvent);
                    } catch (const CORBA::Exception& e) {
                        cerr << "Consumer caught CORBA::Exception: " <<
                                e << endl;
                        return;
                    }
                }
                cout << "Consumer got event" << endl;
                if (*anyVal >>= longVal) {
                    cout << "Consumer pulled long: " << longVal << endl;
                    if (longVal == 13) {
                        cout << "Consumer received #13, exiting..." <<
                                endl;
                        _pullSupplier->disconnect_pull_supplier();
                        return;
                    }
                } else if (*anyVal >>= shortVal) {
                    cout << "Consumer pulled short: " << shortVal <<
                            endl;
                } else if (*anyVal >>= doubleVal) {
                    cout << setiosflags(ios::fixed);
                    cout << "Consumer: pulled double: " << doubleVal <<
                            endl;
                } else if (*anyVal >>= stringVal) {
                    cout << "Consumer: pulled string: " << stringVal <<
                            endl;
                    CORBA::string_free(stringVal);
                }
                hasEvent = 0;
            } catch (const CosEventComm::Disconnected& e) {
                cerr << "Consumer caught Disconnected Exception" <<
                        endl;
                return;
            } catch (const CORBA::Exception& e) {
                cerr << "Consumer caught CORBA::Exception: " << e <<
                        endl;
                return;
            }
        }
    }

    void disconnect_pull_consumer() {

        cout << "PurePullConsumer::disconnect_pull_consumer()" << endl;

        try {
            PortableServer::ObjectId_var objId = PortableServer::
                    string_to_ObjectId("PurePullConsumer");
            _poa->deactivate_object(objId);
        } catch (const CORBA::Exception& e) {
            cout << e << endl;
        }
    }

private:
    PortableServer::POA_var _poa;
    CosEventComm::PullSupplier_var _pullSupplier;
};

int main(int argc, char** argv) {

    // initialize the ORB
    CORBA::ORB_var orb = CORBA::ORB_init(argc, argv);

    // 1. The client (Supplier or Consumer) first binds to the Event
    // Channel (which must be created by someone already, perhaps the
    // client itself).
    CORBA::Object_var obj = orb->
            resolve_initial_references("EventService");
    CosEventChannelAdmin::EventChannel_var eventChannel =
            CosEventChannelAdmin::EventChannel::_narrow(obj);
    if (eventChannel == NULL) {
        cerr << "could not find EventChannel" << endl;
        return -1;
    }
    cout << "found the EventChannel" << endl;

    // 2. The client obtains an Admin object from the Event Channel. A
    // Consumer needs a ConsumerAdmin object, and a Supplier needs a
    // SupplierAdmin object.
    CosEventChannelAdmin::ConsumerAdmin_var consumerAdmin =
            eventChannel->for_consumers();

    // 3. The client obtains a proxy object from the Admin object (a
    // Consumer Proxy for a Supplier client, and a Supplier Proxy for a
    // Consumer client).
    CosEventChannelAdmin::ProxyPullSupplier_var proxySupplier =
            consumerAdmin->obtain_pull_supplier();
    cout << "got a ProxyPullSupplier" << endl;

    // obtain a reference to the Root POA
    obj = orb->resolve_initial_references("RootPOA");
    PortableServer::POA_var rootPOA = PortableServer::POA::_narrow(obj);

    // create the policies for the push supplier POA
    CORBA::PolicyList policies;
    policies.length(1);
    policies[0] = rootPOA->create_lifespan_policy(PortableServer::
            PERSISTENT);

    // create the push supplier POA
    PortableServer::POAManager_var poaManager = rootPOA->
            the_POAManager();
    PortableServer::POA_var consumerPOA = rootPOA->
            create_POA("PullConsumerPOA", poaManager, policies);

    // create and activate the push supplier
    PurePullConsumer* pullConsumer = new PurePullConsumer(proxySupplier,
            consumerPOA);
    CORBA::String_var consumerName(CORBA::
            string_dup("PurePullConsumer"));
    PortableServer::ObjectId_var objId = PortableServer::
            string_to_ObjectId(consumerName);
    consumerPOA->activate_object_with_id(objId, pullConsumer);
    consumerPOA->the_POAManager()->activate();
    consumerPOA->servant_to_reference(pullConsumer);

    // 4. The client adds the Supplier or Consumer to the Event Channel
    // via a connect() call.
    proxySupplier->connect_pull_consumer(pullConsumer->_this());

    // 5. The client and/or the Event Channel transfer data via the\
    // push(), pull(), and/or try_pull() calls.
    pullConsumer->start();

    cout << "Consumer is ending" << endl;
    return 0;
}
