// C++

#include <unistd.h>
#include <iostream.h>
#include <corba.h>
#include <CosEventComm_s.hh>
#include <CosEventChannelAdmin_c.hh>

class PurePushSupplier : public POA_CosEventComm::PushSupplier {

public:
    PurePushSupplier(CosEventComm::PushConsumer_ptr pushConsumer,
            PortableServer::POA_ptr poa) : _pushConsumer(pushConsumer),
            _poa(poa) { }

    void start() {

        CORBA::Any any;

        // push some events to the channel

        any <<= (CORBA::ULong)555555555;
        _pushConsumer->push(any);
        cout << "Supplier: just pushed a long value" << endl;
        sleep(1);

        any <<= (CORBA::Short)100;
        _pushConsumer->push(any);
        cout << "Supplier: just pushed a short value" << endl;

        any <<= (CORBA::Double)999999.999999;
        _pushConsumer->push(any);
        cout << "Supplier: just pushed a double value" << endl;

        CORBA::String_var str = CORBA::string_dup("And the ubiquitous "
                "Hello World!");
        any <<= str;
        _pushConsumer->push(any);
        cout << "Supplier: just pushed a string" << endl;
        sleep(5);

        any <<= (CORBA::ULong)13;
        _pushConsumer->push(any);

        _pushConsumer->disconnect_push_consumer();
    }

    void disconnect_push_supplier() {

        cout << "PurePushSupplier::disconnect_push_supplier()" << endl;

        try {
            PortableServer::ObjectId_var objId = PortableServer::
                    string_to_ObjectId("PurePushSupplier");
            _poa->deactivate_object(objId);
        } catch (const CORBA::Exception& e) {
            cout << e << endl;
        }
    }

private:
    PortableServer::POA_var _poa;
    CosEventComm::PushConsumer_var _pushConsumer;
};

int main(int argc, char** argv) {

    // initialize the ORB
    CORBA::ORB_var orb = CORBA::ORB_init(argc, argv);

    // 1. The client (Supplier or Consumer) first binds to the Event
    // Channel (which must be created by someone already, perhaps the
    // client itself).
    CORBA::Object_var obj = orb->
            resolve_initial_references("EventService");
    CosEventChannelAdmin::EventChannel_var eventChannel =
            CosEventChannelAdmin::EventChannel::_narrow(obj);
    if (eventChannel == NULL) {
        cerr << "could not find EventChannel" << endl;
        return -1;
    }
    cout << "found the EventChannel" << endl;

    // 2. The client obtains an Admin object from the Event Channel. A
    // Consumer needs a ConsumerAdmin object, and a Supplier needs a
    // SupplierAdmin object.
    CosEventChannelAdmin::SupplierAdmin_var supplierAdmin =
            eventChannel->for_suppliers();

    // 3. The client obtains a proxy object from the Admin object (a
    // Consumer Proxy for a Supplier client, and a Supplier Proxy for a
    // Consumer client).
    CosEventChannelAdmin::ProxyPushConsumer_var proxyConsumer =
            supplierAdmin->obtain_push_consumer();
    cout << "got a ProxyPushConsumer" << endl;

    // obtain a reference to the Root POA
    obj = orb->resolve_initial_references("RootPOA");
    PortableServer::POA_var rootPOA = PortableServer::POA::_narrow(obj);

    // create the policies for the push supplier POA
    CORBA::PolicyList policies;
    policies.length(1);
    policies[0] = rootPOA->create_lifespan_policy(PortableServer::
            PERSISTENT);

    // create the push supplier POA
    PortableServer::POAManager_var poaManager = rootPOA->
            the_POAManager();
    PortableServer::POA_var supplierPOA = rootPOA->
            create_POA("pushSupplierPOA", poaManager, policies);

    // create and activate the push supplier
    PurePushSupplier* pushSupplier = new PurePushSupplier(proxyConsumer,
            supplierPOA);
    CORBA::String_var supplierName(CORBA::
            string_dup("PurePushSupplier"));
    PortableServer::ObjectId_var objId = PortableServer::
            string_to_ObjectId(supplierName);
    supplierPOA->activate_object_with_id(objId, pushSupplier);
    supplierPOA->the_POAManager()->activate();
    supplierPOA->servant_to_reference(pushSupplier);

    // 4. The client adds the Supplier or Consumer to the Event Channel
    // via a connect() call.
    proxyConsumer->connect_push_supplier(pushSupplier->_this());

    // 5. The client and/or the Event Channel transfer data via the
    // push(), pull(), and/or try_pull() calls.
    pushSupplier->start();

    cout << "Supplier is ending" << endl;
    return 0;
}
